#!/bin/bash
#
# Driver Uninstall Script
# Copyright (c) 2005-2021 Kodak Alaris Inc.
#
# Parameters:
# $1 - true        remove model specific user folder (default)
#      false       do not remove model specific user folder
#
#
#
# Version
# --------------------------------------------------------------------
# 1.5   12/15/20 - re-write and move items to .spec file
# 1.6   02/08/21 - only remove TWAIN DSM and OpenUSB if no drivers for anyone



################################################################################
# Initialize product specific variables
################################################################################
dialog="KODAK S2000 Scanners"
model=s2000
modeldir=kds_s2000
modelpkgname=kodak_s2000
mfgdir=kodak
versioncd=7.8
vids=( 29cc )
drvignoredirs="" # NOTE: this list in cd_linux.spec also
drvignoredirs="${drvignoredirs} common"
drvignoredirs="${drvignoredirs} deviceinfo"
drvignoredirs="${drvignoredirs} kascannerservice"
drvignoredirs="${drvignoredirs} setup*"
drvignoredirs="${drvignoredirs} twain"
drvignoredirs="${drvignoredirs} install*"



################################################################################
# Check for command-line options
################################################################################
removeuserdir=$1
if [ "$removeuserdir" == "" ]; then
	removeuserdir="true"
fi



################################################################################
# remove the given directory if it is has no scanners in it
################################################################################
function remove_empty_dir()
{
	# see if directory is even there
	if [ ! -d "$1" ]; then
		return
	fi

	# let's switch to the directory incase it has a space, so we can do wildcard compares
	olddir=`pwd`
	cd "$1"

	# look at everything in the folder
	isempty="1" # assume it is empty
	dirfilelist=`ls`
	for file in ${dirfilelist}
	do
		# we don't want to see any files
		if [ -f "$file" ]; then
			echo "   file '$1/$drvignoredir' exists, so not removing '$1'"
			isempty="0"
			break
		fi
		
		# see if it is a directory we can ignore
		canskip="0"
		for drvignoredir in ${drvignoredirs}
		do
			# see if it matches a dir we can ignore
			# NOTE: do no double-quote $drvignoredir, otherwise any directories with '*' won't work
			if [ "$file" = "$drvignoredir" ]; then
				echo "   safe to remove directory '$1/$drvignoredir'"
				canskip="1"
				break
			fi
		done
		if [ "$canskip" = "1" ]; then
			continue;
		fi

		# not a directory we can ignore (i.e. driver or an app)
		echo "   directory '$1/$file' exists, so not removing '$1'"
		isempty="0"
		break
	done

	# go back to the original directory
	cd $olddir

	# if 'empty', then remove it
	if [ "$isempty" = "1" ]; then
		echo "   removing directory '$1'"
		rm -fr "$1"
	fi
}



################################################################################
# remove/uninstall the given package
################################################################################
function remove_package()
{
	# see if distro has dpkg
	dpkgpath=`which dpkg 2>> /dev/null`
	if [ "$dpkgpath" != "" ]; then
		if [ "`dpkg --get-selections $1 2>>/dev/null | grep ^$1`" != "" ]; then
			echo "Uninstalling package: $1"
			$dpkgpath -P $1
			return
		fi

		# package name in .spec may have underscore, but alien seems to change it to a dash
		pkgname=`echo $1 | sed -e "s/_/-/g"`
		if [ "`dpkg --get-selections $pkgname 2>>/dev/null | grep ^$pkgname`" != "" ]; then
			echo "Uninstalling package: $pkgname"
			$dpkgpath -P $pkgname
			return
		fi

		# it seems alien may also convert to lower-case, so we will try that next as well
		pkgname=`echo "$pkgname" | tr '[:upper:]' '[:lower:]'`
		echo $pkgname
		if [ "`dpkg --get-selections $pkgname 2>>/dev/null | grep ^$pkgname`" != "" ]; then
			echo "Uninstalling package: $pkgname"
			$dpkgpath -P $pkgname
			return
		fi

		return
	fi

	# see if distro has rpm
	rpmpath=`which rpm 2>> /dev/null`
	if [ "$rpmpath" != "" ]; then
		if [ "`$rpmpath -qa | grep $1-`" != "" ]; then
			echo "Uninstalling package: $1"
			$rpmpath -e --nodeps $1
		fi
		return
	fi

	echo "Uninstalling did not occur for package: $1"
}



################################################################################
# main
################################################################################
# make sure the user is root
if [ `id -u` != "0" ]; then
	echo ""
	echo ""
	echo ""
	echo "*** ERROR ***"
	echo "You must be logged into the root account (i.e. super user)"
	echo ""
	exit
fi


# determine the uninstall program to use
dpkgpath=`which dpkg 2>> /dev/null`
if [ "$dpkgpath" == "" ]; then
	rpmpath=`which rpm 2>> /dev/null`
	if [ "$rpmpath" == "" ]; then
		echo ""
		echo ""
		echo ""
		echo "*** ERROR ***"
		echo "Package installer (e.g. dpkg/rpm) is not installed. Unable to uninstall driver."
		echo ""
		exit
	fi
fi


# make sure the scanner is disconnected
echo ""
echo ""
echo ""
echo "****************************************************************************"
echo "Turn off all scanners and unplug all scanner USB cables from your computer"
echo ""
echo "Press Enter to continue..."
read readPrompt
echo ""
echo ""
echo ""
echo "****************************************************************************"
echo "Removing software, please wait..."
echo ""


# we can't be in /opt if we're going to be deleting its contents. So, save
# the current folder and change the working directory to 'home'.
_saved_dir=`pwd`
cd ~


# if caller wants to leave the user data, then set environment variable while package is removed
if [ "$removeuserdir" == "false" ]; then
	echo "Asked to keep user data"
	DRV_UNINSTALL_KEEP_USERDIR="false"
	export DRV_UNINSTALL_KEEP_USERDIR
fi



################################################################################
# remove ADF (NOTE: no more separate packages for included flatbeds)
################################################################################
remove_package $modelpkgname



################################################################################
# remove any empty manufacturer folders
################################################################################
# NOTE: while this is in .spec, we still want this incase other clean up occurred
remove_empty_dir "/var/$mfgdir"
remove_empty_dir "/opt/$mfgdir"



################################################################################
# if no more drivers for this manufacturer, then remove all the shared stuff
################################################################################
if [ ! -d "/opt/$mfgdir" ]; then
	echo "   Manufacturer '$mfgdir' no longer has installed items"

	# remove the rules
	# NOTE: while this is in .spec, we still want this incase other clean up occurred
	rm -fr /etc/udev/rules.d/55-${mfgdir}di.rules

	# removed shared packages. now that OpenUSB and TWAIN DSM are in the
	# driver's lib folder, these packages could only part of the setup script
	# or any old install. TWAIN DSM is part of setup script to support apps, so
	# we can only remove if there are no other data sources installed. and that
	# is for any manufacturer, as we don't want to break others. OpenUSB would
	# only be there is leftover. we will just play it safe and only remove it
	# if no other drivers installed. again, dont' want to break others.
	anydrivers=`ls -R /usr/local/lib/twain | grep -i "\.ds"`
	if [ "$anydrivers" != "" ]; then
		echo "   Found another driver. TWAIN DSM will not be removed."
	else
		echo "   No drivers found. TWAIN DSM will be removed."
		for unpkgname in libopenusb twaindsm
		do
			remove_package $unpkgname
		done
	fi
fi


# undo flag for leaving user data
if [ "$removeuserdir" == "false" ]; then
	DRV_UNINSTALL_KEEP_USERDIR=
	export DRV_UNINSTALL_KEEP_USERDIR
fi


# if the folder we started in still exists, then return to it
cd "$_saved_dir" > /dev/null 2>&1


# if the starting folder has been deleted, tell the user
_new_dir=`pwd`
if [ "$_new_dir" != "$_saved_dir" ]; then
	echo ""
	echo ""
	echo ""
	echo "****************************************************************************"
	echo "The starting folder '$_saved_dir' has been removed."
	echo "Please cd to an existing folder..."
	echo ""
fi


echo ""
echo ""
echo ""
echo "****************************************************************************"
echo "Uninstall is complete"
echo ""
